#!/usr/bin/env python
"""
Simple program allowing the interactive exploration of the
value space of a planar image. 

Based in part on code written by Nicolas Guarin Zapata in 2014
"""

from PyQt4 import Qt, QtCore, QtGui
import vtk
from vtk.qt4.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor
import sys

line_width = 1
line_color = { "R": 0.75, "G": 0.5, "B": 0. }

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("QVTKRenderWindowInteractor")
        MainWindow.setWindowTitle('Value Browser')
        self.centralWidget = QtGui.QWidget(MainWindow)
        self.gridlayout = QtGui.QGridLayout(self.centralWidget)
        self.vtkWidget = QVTKRenderWindowInteractor(self.centralWidget)
        self.gridlayout.addWidget(self.vtkWidget, 0, 0, 1, 9)
        # Instantiate the sliders
        self.sliderStart = QtGui.QSlider() 
        self.sliderEnd = QtGui.QSlider() 
        self.sliderWidth = QtGui.QSlider()
        self.startValueLabel = QtGui.QLabel("dummy")
        self.endValueLabel = QtGui.QLabel("dummy")
        self.widthValueLabel = QtGui.QLabel("dummy")
        self.gridlayout.addWidget(QtGui.QLabel("Start"), 1, 0)
        self.gridlayout.addWidget(self.sliderStart, 1, 1)
        self.gridlayout.addWidget(self.startValueLabel, 1, 2)
        self.gridlayout.addWidget(QtGui.QLabel("End"), 1, 3)
        self.gridlayout.addWidget(self.sliderEnd, 1, 4)
        self.gridlayout.addWidget(self.endValueLabel, 1, 5)
        self.gridlayout.addWidget(QtGui.QLabel("Width"), 1, 6)
        self.gridlayout.addWidget(self.sliderWidth, 1, 7) 
        self.gridlayout.addWidget(self.widthValueLabel, 1, 8)
        MainWindow.setCentralWidget(self.centralWidget)
 
class ValueBrowser(QtGui.QMainWindow):
 
    def __init__(self, filename, parent = None):
        QtGui.QMainWindow.__init__(self, parent)
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        
        # Import file
        reader = vtk.vtkDataSetReader()
        reader.SetFileName(filename)
        reader.Update()
        
        range = reader.GetOutput().GetPointData().GetScalars().GetRange()
        
        print "input values range from " + str(range[0]) + \
                " to " + str(range[1])
        self.min = range[0]
        self.max = range[1]
        self.start = 0.75*range[0]+0.25*range[1]
        self.end = 0.25*range[0]+0.75*range[1]
        self.width = 0.05*(range[1]-range[0])
        self.ui.startValueLabel.setText(str(self.start))
        self.ui.endValueLabel.setText(str(self.end))
        self.ui.widthValueLabel.setText(str(self.width))
                       
        self.grayTF = vtk.vtkColorTransferFunction()
        self.grayTF.AddRGBPoint(range[0], 0, 0, 0) # global min -> black
        self.grayTF.AddRGBPoint(range[1], 1, 1, 1) # global max -> white
                       
        self.colorTF = vtk.vtkColorTransferFunction()
                       
        color_mapper = vtk.vtkDataSetMapper()
        color_mapper.SetInputConnection(reader.GetOutputPort())
        color_mapper.ScalarVisibilityOn()
        color_mapper.SetLookupTable(self.colorTF)
        color_actor = vtk.vtkActor()
        color_actor.SetMapper(color_mapper)
        
        # Show level sets corresponding to the interactive control points
        self.contour = vtk.vtkContourFilter()
        self.contour.SetInputConnection(reader.GetOutputPort())
        self.contour.SetValue(0, self.start-self.width)
        self.contour.SetValue(1, self.start)
        self.contour.SetValue(2, self.end)
        self.contour.SetValue(3, self.end+self.width)
        
        contour_mapper = vtk.vtkPolyDataMapper()
        contour_mapper.SetInputConnection(self.contour.GetOutputPort())
        contour_mapper.ScalarVisibilityOff()
        contour_actor = vtk.vtkActor()
        contour_actor.SetMapper(contour_mapper)
        contour_actor.GetProperty().SetColor(line_color["R"], \
            line_color["G"], line_color["B"])
        contour_actor.GetProperty().SetLineWidth(line_width)
        
        # Create a 2D image style for mouse interaction
        image_style = vtk.vtkInteractorStyleImage()
        image_style.SetInteractionModeToImage2D()
        
        # Create the Renderer
        self.ren = vtk.vtkRenderer()
        self.ren.AddActor(color_actor)
        self.ren.AddActor(contour_actor)
        self.ren.GradientBackgroundOn()  # Set gradient for background
        self.ren.SetBackground(0.75, 0.75, 0.75)  # Set background to silver
        self.ui.vtkWidget.GetRenderWindow().AddRenderer(self.ren)
        self.iren = self.ui.vtkWidget.GetRenderWindow().GetInteractor()
        self.iren.SetInteractorStyle(image_style)
                
        # Setting up widgets
        def slider_setup(slider, val, bounds, interv):
            slider.setOrientation(QtCore.Qt.Horizontal)
            slider.setValue(float(val))
            slider.setTracking(False)
            slider.setTickInterval(interv)
            slider.setTickPosition(QtGui.QSlider.TicksAbove)
            slider.setRange(bounds[0], bounds[1])
            
        slider_setup(self.ui.sliderStart, 25, [0, 100], 10)
        slider_setup(self.ui.sliderEnd, 75, [0, 100], 10)
        slider_setup(self.ui.sliderWidth, 5, [0, 100], 10)
        
        self.updateColorTransferFunction()
        
    def updateColorTransferFunction(self):
        self.colorTF.RemoveAllPoints()
        self.colorTF.AddRGBPoint(self.min, 0, 0, 0) # min value -> black
        self.colorTF.AddRGBPoint(self.max, 1, 1, 1) # max value -> white
        self.colorTF.AddRGBPoint(self.start, 1, 0, 0) # start of red region
        self.colorTF.AddRGBPoint(self.end, 1, 0, 0) # end of red region
        # make sure that the control points remain in the correct order
        low=max(self.min, self.start-self.width)
        if low<self.start:
            rgb = self.grayTF.GetColor(low)
            self.colorTF.AddRGBPoint(low, rgb[0], rgb[1], rgb[2])
        high=min(self.max, self.end+self.width)
        if high>self.end:
            rgb = self.grayTF.GetColor(high)
            self.colorTF.AddRGBPoint(high, rgb[0], rgb[1], rgb[2])
        # it is now time to render
        self.ui.vtkWidget.GetRenderWindow().Render() 
        
    def pct2Value(self, pct):
        u = float(pct)/100.
        val = (1.-u)*self.min + u*self.max
        return val
         
    def start_callback(self, pct):
        # enforce start<end invariant
        val = self.pct2Value(pct)
        if val>self.end:
            self.start = self.end
            self.ui.sliderStart.setValue(self.ui.sliderEnd.value())
        else:
            self.start = val
        self.contour.SetValue(1, self.start)
        self.ui.startValueLabel.setText(str(self.start))
        self.updateColorTransferFunction()
        
    def end_callback(self, pct):
        # enforce start<end invariant
        val = self.pct2Value(pct)
        if val<self.start:
            self.end = self.start
            self.ui.sliderEnd.setValue(self.ui.sliderStart.value())
        else:
            self.end = val
        self.contour.SetValue(2, self.end)
        self.ui.endValueLabel.setText(str(self.end))
        self.updateColorTransferFunction()

    def width_callback(self, pct):
        self.width = float(pct)/100.*(self.max-self.min)
        self.contour.SetValue(0, self.start-self.width)
        self.contour.SetValue(3, self.end+self.width)
        self.ui.widthValueLabel.setText(str(self.width))
        self.updateColorTransferFunction()

def PrintUsage(msg):
    if len(msg) > 0:
        print "ERROR: " + msg
    print "USAGE: " + sys.argv[0] + " <filename> [options]"        
    print "DESCRIPTION: Value browsing utility using interactive"
    print "             color maps."
    print "PARAMETER:"
    print " <filename>: name of VTK dataset corresponding to a 2D scalar image"
    print "OPTION:"
    print " -h | --help                  Print this information"
    print " -w | --width <value>         Specify line width for isocontours"
    print " -c | --color <R> <G> <B>     Specify color for isocontours"
    sys.exit()


if __name__ == "__main__":
    
    N = len(sys.argv)-1
    if N == 0:
        PrintUsage("Missing dataset filename")
    filename = sys.argv[1]
    i=1
    while i < N:
        i += 1
        arg = sys.argv[i]
        if arg == "-h" or arg == "--help":
            PrintUsage("")
        elif arg == "-w" or arg == "--width":
            if N == i:
                PrintUsage("Missing <width> value")
            line_width = int(sys.argv[i+1])
            i += 1
        elif arg == "-c" or arg == "--color":
            if i > N-3:
                PrintUsage("Missing / incomplete color definition")
            line_color = { "R": float(sys.argv[i+1]), \
                           "G": float(sys.argv[i+2]), \
                           "B": float(sys.argv[i+3]) }
            i += 3
        else:
            PrintUsage("Unrecognized input argument")
    
    app = QtGui.QApplication(sys.argv)
    window = ValueBrowser(filename)
    window.show()
    window.setWindowState(QtCore.Qt.WindowMaximized)  # Maximize the window
    window.iren.Initialize() # Need this line to actually show
                             # the renderer inside Qt
    app.connect(window.ui.sliderStart, Qt.SIGNAL('valueChanged(int)'),
                window.start_callback)
                
    app.connect(window.ui.sliderEnd, Qt.SIGNAL('valueChanged(int)'),
                window.end_callback)
                
    app.connect(window.ui.sliderWidth, Qt.SIGNAL('valueChanged(int)'),
                window.width_callback)
    sys.exit(app.exec_())
    
