#!/usr/bin/env python

import sys
import vtk
import os
from os import path

EPSILON  = 1.0e-6

# convert (global) values to their position relative 
# to the min (gmin) and max (gmax) values of the dataset
def g2l(v):
    global gmin, gmax
    return (float(v)-gmin)/(gmax-gmin);

# Set the control points of the color transfer function
# ctf: color transfer function
# smin, smax: value interval to show in red in the visualization
# gmin, gmax: global min and max values
# width: width of the up and down transition ramp between red and gray
def setCTF():
    global ctf, smin, smax, gmin, gmax, width
    
    # we start by removing all the previous control points
    ctf.RemoveAllPoints()
    
    # local coordinates of ramps' end points
    ulo = max(0, g2l(smin-width))
    uhi = min(1, g2l(smax+width))
    # lower half of color map
    if (smin > gmin):
        ctf.AddRGBPoint(gmin, 0, 0, 0)
        ctf.AddRGBPoint(smin, 1, 0, 0)
        if (smin-width > gmin): 
            ctf.AddRGBPoint(smin-width, ulo, ulo, ulo)
    else:
         ctf.AddRGBPoint(gmin, 1, 0, 0);
    # upper half of color map
    if (smax < gmax):
        ctf.AddRGBPoint(smax, 1, 0, 0)
        ctf.AddRGBPoint(gmax, 1, 1, 1)
        if (smax+width < gmax):
            ctf.AddRGBPoint(smax+width, uhi, uhi, uhi)
    else:
        ctf.AddRGBPoint(gmax, 1, 0, 0);

def SliderCallbackMin(obj, event):
    global smin, smax
    
    rep = obj.GetRepresentation()
    val = rep.GetValue()
    if val > smax:
        # enforce smin <= smax
        smin = smax
        rep.SetValue(smin)
    else:
        smin = val
    setCTF()
    
def SliderCallbackMax(obj, event):
    global smin, smax
    
    rep = obj.GetRepresentation()
    val = rep.GetValue()
    if val < smin:
        # enforce smin <= smax
        smax = smin
        rep.SetValue(smax)
    else:
        smax = val
    setCTF()

def SliderCallbackWidth(obj, event):
    global width

    rep = obj.GetRepresentation()
    width = rep.GetValue()
    setCTF()


# Simple function to create a vtkSliderRepresentation2D. We pack this code
# in a function to avoid writing it twice since it is quite painful...
def makeRep(text, vmin, vmax, v, x, dx):
    rep = vtk.vtkSliderRepresentation2D()
    rep.SetMinimumValue(vmin)
    rep.SetMaximumValue(vmax)
    rep.SetValue(v)
    rep.SetTitleText(text)
    rep.SetLabelFormat("%0.0f")
    rep.GetPoint1Coordinate().SetCoordinateSystemToNormalizedDisplay()
    rep.GetPoint1Coordinate().SetValue(x, 0.07)
    rep.GetPoint2Coordinate().SetCoordinateSystemToNormalizedDisplay()
    rep.GetPoint2Coordinate().SetValue(x+dx, 0.07)
    rep.SetSliderLength(0.005)
    rep.SetSliderWidth(0.02)
    rep.SetEndCapLength(0.002)
    rep.SetEndCapWidth(0.03)
    rep.SetTubeWidth(0.002)
    rep.GetTubeProperty().SetColor(0.9, 0.9, 0.9)
    rep.GetCapProperty().SetColor(0.9, 0.9, 0.9)
    rep.GetSliderProperty().SetColor(1, 1, 0)
    rep.GetTitleProperty().SetColor(1, 1, 1)
    rep.GetTitleProperty().ShadowOff()
    rep.GetTitleProperty().SetFontFamilyToTimes()
    rep.SetTitleHeight(0.02)
    rep.GetTitleProperty().BoldOff()
    rep.GetLabelProperty().SetColor(1, 1, 1)
    rep.SetLabelHeight(0.02)
    rep.GetLabelProperty().SetFontFamilyToTimes()
    rep.GetLabelProperty().BoldOff()
    rep.GetLabelProperty().ShadowOff()
    return rep

def printUsage(msg):
    global me
    
    if len(msg) > 0:
        print "ERROR: " + msg
    print "USAGE: " + me + " [parameters] [options]"        
    print "DESCRIPTION: Gray scale visualization of scalar dataset with"
    print "             emphasis on selected interval in red."
    print "PARAMETERS:"    
    print " -i | --input <string>   Input file name"
    print "OPTIONS:"
    print " -h | --help             Print this information"
    print " -r | --res <int> (x2)   Render window resolution (default: 800 800)"
    print " -v | --verbose          Turn verbose mode on"
    sys.exit()

def main():
    global me, gmin, gmax, smin, smax, width
    
    verbose = False
    input = "undefined"
    me = sys.argv[0]
    w = 800
    h = 800
    i = 0
    while (i<len(sys.argv)-1):
        i += 1
        arg = sys.argv[i]
        if (arg == "-h" or arg == "--help"):
            printUsage("")
        elif (arg == "-i" or arg == "--input"):
            if (i == len(sys.argv)-1):
                printUsage("missing input name")
            input = sys.argv[i+1]
            i += 1
        elif (arg == "-r" or arg == "--res"):
            if (i >= len(sys.argv)-2):
                printUsage("missing resolution parameters")
            w = sys.argv[i+1]
            h = sys.argv[i+2]
            i += 2
        elif (arg == "-v" or arg == "--verbose"):
            verbose = True
        else:
            printUsage("unrecognized input argument")

    if input == "undefined":
        printUsage("No input data file provided")

    # Import dataset
    reader = vtk.vtkDataSetReader()
    reader.SetFileName(input)
    reader.Update()
    
    if verbose:
        print input + " successfully imported"

    # Determine min and max value in input
    range = reader.GetOutput().GetPointData().GetScalars().GetRange()
    gmin = range[0]
    gmax = range[1]
    if (verbose):
        print "Values range from " + str(gmin) + " to " + str(gmax)

    # Initialize smin and smax to mid point
    smin = smax = 0.5*(gmin + gmax)
    width = 0.05*(gmax - gmin)

    # Initialize color map
    global ctf
    ctf = vtk.vtkColorTransferFunction()
    setCTF()

    # Create a color bar
    scalar_bar = vtk.vtkScalarBarActor()
    scalar_bar.SetLookupTable(ctf)
    scalar_bar.SetPosition(0.85, 0.75)
    scalar_bar.SetMaximumWidthInPixels(80)
    scalar_bar.SetMaximumHeightInPixels(25*h/100-20)
    scalar_bar.SetNumberOfLabels(5)
    scalar_bar.SetLabelFormat("%0.0f")
    scalar_bar.SetTitle("Color map")
    scalar_bar.GetTitleTextProperty().SetColor(1, 1, 1)
    scalar_bar.GetTitleTextProperty().ShadowOff()
    scalar_bar.GetTitleTextProperty().BoldOn()
    scalar_bar.GetLabelTextProperty().SetColor(1, 1, 1)
    scalar_bar.GetLabelTextProperty().BoldOff()
    scalar_bar.GetLabelTextProperty().ShadowOff()

    # Usual rendering stuff...
    mapper = vtk.vtkDataSetMapper()
    mapper.SetInputConnection(reader.GetOutputPort())
    mapper.SetLookupTable(ctf)
    # actor
    actor = vtk.vtkActor()
    actor.SetMapper(mapper)
    # renderer
    renderer = vtk.vtkRenderer()
    renderer.AddActor(actor)
    renderer.AddActor(scalar_bar)
    renderer.SetBackground(0, 0, 0) # black background
    renderer.ResetCamera()
    renderer.LightFollowCameraOn()
    # render window
    window = vtk.vtkRenderWindow()
    window.AddRenderer(renderer)
    window.SetSize(w, h)
    # interactor
    interactor = vtk.vtkRenderWindowInteractor()
    img_style = vtk.vtkInteractorStyleImage()
    interactor.SetInteractorStyle(img_style)
    interactor.SetRenderWindow(window)
    window.Render()

    # Widget stuff
    # min value slider
    rep_min = makeRep("Min Value", gmin, gmax, smin, 0.025, 0.3) 
    widget_min = vtk.vtkSliderWidget()
    widget_min.SetInteractor(interactor)
    widget_min.SetRepresentation(rep_min)
    widget_min.SetAnimationModeToJump()
    widget_min.AddObserver("InteractionEvent", SliderCallbackMin)
    widget_min.EnabledOn()
    # max value slider
    rep_max = makeRep("Max Value", gmin, gmax, smax, 0.35, 0.3)    
    widget_max = vtk.vtkSliderWidget()
    widget_max.SetInteractor(interactor)
    widget_max.SetRepresentation(rep_max)
    widget_max.SetAnimationModeToJump()
    widget_max.AddObserver("InteractionEvent", SliderCallbackMax)
    widget_max.EnabledOn()
    # ramp width slider
    rep_width = makeRep("Max Value", 0.001, 0.25*(gmax-gmin), width, 0.675, 0.3)    
    widget_width = vtk.vtkSliderWidget()
    widget_width.SetInteractor(interactor)
    widget_width.SetRepresentation(rep_width)
    widget_width.SetAnimationModeToJump()
    widget_width.AddObserver("InteractionEvent", SliderCallbackWidth)
    widget_width.EnabledOn()

    if verbose: 
         print "Entering interactive mode..."
    
    interactor.Initialize()
    interactor.Start()

if __name__=="__main__":
  main()
